# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Word Trend Analysis
#
# ### Load previously vectorized corpus
#
# Use the `vectorize_protocol` script to create a new corpus with different settings.

# %%
# %load_ext autoreload
# %autoreload 2

# %%

import importlib

import penelope.notebook.load_vectorized_corpus_gui as load_corpus_gui
import penelope.notebook.utility as notebook_utility
import penelope.notebook.vectorize_corpus_gui as vectorize_corpus_gui
from bokeh.plotting import output_notebook
from penelope.notebook.word_trends.loaded_callback import loaded_callback

output_notebook(hide_banner=True)

# %% tags=[] vscode={"end_execution_time": "2020-08-31T18:34:55.995Z", "start_execution_time": "2020-08-31T18:34:55.854Z"}
importlib.reload(vectorize_corpus_gui)
importlib.reload(notebook_utility)
importlib.reload(load_corpus_gui)
importlib.reload(notebook_utility)

# %% tags=[] vscode={"end_execution_time": "2020-08-31T18:34:55.995Z", "start_execution_time": "2020-08-31T18:34:55.854Z"}
importlib.reload(vectorize_corpus_gui)

data = None


def loaded_callback_wrapper(
    output,
    *,
    corpus=None,
    corpus_folder: str = None,
    corpus_tag: str = None,
    n_count: int = 10000,
    **kwargs,
):
    global data
    data = locals()
    loaded_callback(
        output, corpus=corpus, corpus_folder=corpus_folder, corpus_tag=corpus_tag, n_count=n_count, **kwargs
    )


vectorize_corpus_gui.display_gui('*sparv4.csv.zip', generated_callback=loaded_callback_wrapper)


# %%
load_corpus_gui.display_gui(loaded_callback=loaded_callback)

# %%



import math
import warnings

import bokeh
import bokeh.plotting
import numpy as np
import pandas as pd
import penelope.utility as utility

warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)


def _plot(df: pd.DataFrame, category_column: str, value_column: str, x_label=None, y_label=None, **figopts):

    xs = df[category_column].astype(np.str)
    ys = df[value_column]

    y_max = ys.max()  # max(ys.max(), 0.1)

    figopts = utility.extend(dict(title='', toolbar_location="right", y_range=(0.0, y_max)), figopts)

    p = bokeh.plotting.figure(**figopts)

    _ = p.vbar(x=xs, top=ys, width=0.5, fill_color="#b3de69")

    p.xaxis.major_label_orientation = math.pi / 4
    p.xgrid.grid_line_color = None
    p.xaxis[0].axis_label = (x_label or category_column.title().replace('_', ' ')).title()
    p.yaxis[0].axis_label = (y_label or value_column.title().replace('_', ' ')).title()
    # p.y_range.start = 0.0
    p.y_range.start = 0.0
    p.x_range.range_padding = 0.01

    return p


def display(
    weight_over_time: pd.DataFrame,
    topic_id: int,
    year_range,
    aggregate: str,
    output_format: str = 'Chart',
    normalize: bool = True,
):  # pylint: disable=unused-argument

    figopts = dict(plot_width=1000, plot_height=400, title='', toolbar_location="right")

    df = weight_over_time[(weight_over_time.topic_id == topic_id)]

    min_year, max_year = year_range
    figopts['x_range'] = list(map(str, range(min_year, max_year + 1)))

    if len(df) == 0:
        print('No data to display for this topic and theshold')
    elif output_format == 'Table':
        print(df)
    else:
        p = _plot(df, 'year', aggregate, **figopts)
        bokeh.plotting.show(p)



import os
import sys


def find_root_folder(x):
    return os.path.join(os.getcwd().split(x)[0], x)


root_folder = ''

if os.environ.get("JUPYTER_IMAGE_SPEC", "") == "westac_lab":
    root_folder = "/home/jovyan/work/welfare_state_analytics"
else:
    root_folder = find_root_folder("welfare_state_analytics")

if root_folder not in sys.path:
    sys.path.insert(0, root_folder)



from typing import Dict, List, Tuple

import pandas as pd
from penelope.utility import flatten


def setup_pandas():

    pd.set_option("max_rows", None)
    pd.set_option("max_columns", None)
    pd.set_option('colheader_justify', 'left')
    pd.set_option('max_colwidth', 300)


def to_text(document: List[Tuple[int, int]], id2token: Dict[int, str]):
    return ' '.join(flatten([f * [id2token[token_id]] for token_id, f in document]))



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# # Find word's that changes most over time
#
# ## Approach
#
# Compare word's distribution over time with a uniform distribution. Use as null hypothesis the belief that a word's distribution does not change over time. Filter out all the words for which there is no significance.
#
# https://en.wikipedia.org/wiki/Goodness_of_fit#Categorical_data
#
#
# ## Candidate methods
#
# See: https://stats.stackexchange.com/questions/25827/how-does-one-measure-the-non-uniformity-of-a-distribution
#
#
# ### DONE Chi-square test for a discrete uniform distribution
#
# A χ2 goodness-of-fit test is used to determine how (un)likely a data serie (i.e. the word's distribution over time) has been generate by a (discrete) uniform distribution. The actual word counts for each year are used since χ2 is not applicable to relative frequencies. As a rule of thumb, χ2 test requires each individual value to be greater or equal to 5.
#
# From [stats.stackexchange.com](stats.stackexchange.com/questions/25827/how-does-one-measure-the-non-uniformity-of-a-distribution):
#
# *If you have not only the frequencies but the actual counts, you can use a χ2 goodness-of-fit test for each data series. In particular, you wish to use the test for a discrete uniform distribution. This gives you a good test, which allows you to find out which data series are likely not to have been generated by a uniform distribution, but does not provide a measure of uniformity..... (I guess that the chi-squared statistic can be seen as a measure of uniformity, but it has some drawbacks, such as the lack of convergence, dependence on the arbitrarily placed bins, that the number of expected counts in the cells needs to be sufficiently large, etc. Which measure/test to use is a matter of taste though, and entropy is not without its problems either (in particular, there are many different estimators of the entropy of a distribution). To me, entropy seems like a less arbitrary measure and is easier to interpret.)*
#
#  $\tilde{\chi}^2=\frac{1}{d}\sum_{k=1}^{n} \frac{(O_k - E_k)^2}{E_k}$ (d degree of freedom, n samples, E expected, O observed)
#
# References:
#
#   - [Chi-squared test](https://en.wikipedia.org/wiki/Chi-squared_test)
#   - [comparing-two-word-distributions](stats.stackexchange.com/questions/236192/comparing-two-word-distributions)
#
# DONE ### Simple linear regression
#
# Use least-squares fit to compute a Compare word's distribution over time with a uniform distribution. Use as null hypothesis the belief that a word's distribution does not change over time. Filter out all the words for which there is no significance.
#
# | Slope | $y = k * x + m$ | Use linear regression to compute slope k. Select n word having highest absoulute value |
#
# ### G-test for a discrete uniform distribution
#
# en.wikipedia.org/wiki/G-test
#
# ### Kolmogorov-Smirnov test (KS-test)
#
# stackoverflow.com/questions/25208421/how-to-test-for-uniformity
#
# ### Entropy
#
# https://en.wikipedia.org/wiki/Entropy_%28information_theory%29
#
#
# *There are other possible approaches, such as computing the entropy of each series - the uniform distribution maximizes the entropy, so if the entropy is suspiciously low you would conclude that you probably don't have a uniform distribution. That works as a measure of uniformity in some sense.*
#
# ### Kullback-Leibler divergence (KS-test)
# Another suggestion would be to use a measure like the Kullback-Leibler divergence, which measures the similarity of two distributions.
#
#
# ### L2 norm
#
# *Here is a simple heuristic: if you assume elements in any vector sum to 1 (or simply normalize each element with the sum to achieve this), then uniformity can be represented by L2 norm, which ranges from 1d√ to 1, with d being the dimension of vectors. The lower bound 1d√ corresponds to uniformity and upper bound to the 1-hot vector. To scale this to a score between 0 and 1, you can use n∗d√−1d√−1, where n is the L2 norm.
#
# ```
# def gof_by_l2_norm(matrix, axis=1):
#     d = matrix.shape[int(not axis)]
#     l2_norm = (np.linalg.norm(matrix, axis=axis) * math.sqrt(d) - 1 ) / (math.sqrt(d) - 1)
#     return l2_norm
# ```
#
# https://stats.stackexchange.com/questions/248772/why-does-the-l2-norm-heuristic-work-in-measuring-uniformity-of-probability-distr?noredirect=1&lq=1
#
# ### Earth Mover Distance
#
# *The earth mover distance, also known as the Wasserstein metric, measures the distance between two histograms. Essentially, it considers one histogram as a number of piles of dirt and then assesses how much dirt one needs to move and how far (!) to turn this histogram into the other. You would measure the distance between your distribution and a uniform one over the days of the week.* (https://stats.stackexchange.com/a/178187)
#
#
# * [variance - How does one measure the non-uniformity of a distribution? - Cross Validated](https://stats.stackexchange.com/questions/25827/how-does-one-measure-the-non-uniformity-of-a-distribution)
# * [G-test - Wikipedia](https://en.wikipedia.org/wiki/G-test#Relation_to_the_chi-squared_test)
# * [g-test uniform distribution python - Sök på Google](https://www.google.com/search?q=g-test+uniform+distribution+python&oq=g-test+&aqs=chrome.0.69i59l2j69i57j0j69i60l2.3775j0j1&sourceid=chrome&ie=UTF-8)
# * [Python String Format Cookbook – mkaz.blog](https://mkaz.blog/code/python-string-format-cookbook/)
# * [The writing and reporting of assertions in tests](https://docs.pytest.org/en/2.8.7/assert.html)
#
# * [scipy.stats.entropy — SciPy v1.3.1 Reference Guide](https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.entropy.html)
# * [python - Interpreting scipy.stats.entropy values - Stack Overflow](https://stackoverflow.com/questions/26743201/interpreting-scipy-stats-entropy-values)
# * [Plotting data with matplotlib — How to Think Like a Computer Scientist: Learning with Python 3](https://howtothink.readthedocs.io/en/latest/PvL_H.html)
# * [lint-analysis/33-cluster-auto.ipynb at master · davidmcclure/lint-analysis](https://github.com/davidmcclure/lint-analysis/blob/master/notebooks/2017/33-cluster-auto.ipynb)
# * [Search · sm.OLS fit.predict plot](https://github.com/search?p=2&q=sm.OLS+fit.predict+plot&type=Code)
# * [statsmodels.regression.linear_model.OLS — statsmodels](https://www.statsmodels.org/dev/generated/statsmodels.regression.linear_model.OLS.html#statsmodels.regression.linear_model.OLS)
# * [Scikit-learn Normalization mode (L1 vs L2 & Max) - Cross Validated](https://stats.stackexchange.com/questions/225564/scikit-learn-normalization-mode-l1-vs-l2-max)
# * [Pyplot tutorial — Matplotlib 3.1.1 documentation](https://matplotlib.org/3.1.1/tutorials/introductory/pyplot.html)
# * [matplotlib axis range - Sök på Google](https://www.google.com/search?q=matplotlib+axis+range&oq=matplotlib+axis&aqs=chrome.3.69i57j0l5.6542j0j4&sourceid=chrome&ie=UTF-8)
# * [patsy - Describing statistical models in Python — patsy 0.5.1+dev documentation](https://patsy.readthedocs.io/en/latest/)
# * [k-nearest neighbors algorithm - Wikipedia](https://en.wikipedia.org/wiki/K-nearest_neighbors_algorithm)
# * [chi squared - Comparing two word distributions - Cross Validated](https://stats.stackexchange.com/questions/236192/comparing-two-word-distributions)
# * [Chi-squared test - Wikipedia](https://en.wikipedia.org/wiki/Chi-squared_test)
# * [scipy - Calculating probability distribution from time series data in python - Stack Overflow](https://stackoverflow.com/questions/49293019/calculating-probability-distribution-from-time-series-data-in-python)
# * [scipy.stats.gaussian_kde — SciPy v1.3.1 Reference Guide](https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.gaussian_kde.html)
# * [What is the difference between trendline and regression line? - Quora](https://www.quora.com/What-is-the-difference-between-trendline-and-regression-line)
# * [How is the k-nearest neighbor algorithm different from k-means clustering? | Python Tutorial](https://pythonprogramminglanguage.com/How-is-the-k-nearest-neighbor-algorithm-different-from-k-means-clustering/)
# * [[1510.00012] Fast Discrete Distribution Clustering Using Wasserstein Barycenter with Sparse Support](https://arxiv.org/abs/1510.00012)
# * [Exceed Synonyms, Exceed Antonyms | Merriam-Webster Thesaurus](https://www.merriam-webster.com/thesaurus/exceed)
# * [python - Swapping 1 with 0 and 0 with 1 in a Pythonic way - Stack Overflow](https://stackoverflow.com/questions/1779286/swapping-1-with-0-and-0-with-1-in-a-pythonic-way/1779448#1779448)
#
# * [Issues · humlab-sead/sead_query_api](https://github.com/humlab-sead/sead_query_api/issues)
# * [scipy.stats.chi2_contingency — SciPy v0.14.0 Reference Guide](https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.chi2_contingency.html)
# * [Capturing of the stdout/stderr output — pytest documentation](https://docs.pytest.org/en/latest/capture.html)
# * [sm.OLS np matrix - Sök på Google](https://www.google.com/search?q=sm.OLS+np+matrix&oq=sm.OLS+np+matrix&aqs=chrome..69i57.11997j0j4&sourceid=chrome&ie=UTF-8)
# * [scipy.stats.chi2_contingency — SciPy v1.3.1 Reference Guide](https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.chi2_contingency.html#scipy.stats.chi2_contingency)
# * [G-test - Wikipedia](https://en.wikipedia.org/wiki/G-test)
# * [OsloMovers/Web.csproj at bd8a6c91aa957bf8a4857eaccbc4b383c711ddfb · webmasterdevlin/OsloMovers](https://github.com/webmasterdevlin/OsloMovers/blob/bd8a6c91aa957bf8a4857eaccbc4b383c711ddfb/Web/Web.csproj)
#

# %%
# %load_ext autoreload
# %autoreload 2

# pylint: disable=wrong-import-position

import itertools
import sys

import bokeh
import pandas as pd
import penelope.utility as utility
from bokeh.io import output_file, output_notebook, show
from bokeh.plotting import figure
from penelope.common import goodness_of_fit as gof
from penelope.corpus import vectorized_corpus

sys.path = ['/home/roger/source/welfare_state_analytics'] + sys.path


logger = utility.get_logger()
output_notebook()

# %%

v_y_corpus = (
    vectorized_corpus.VectorizedCorpus.load(
        tag='SOU_1945-1989_L0_+N_+S', folder='/home/roger/source/welfare_state_analytics/output'
    )
    .group_by_year()
    .slice_by_n_count(10000)
    .slice_by_n_top(100000)
)

v_ny_corpus = v_y_corpus.normalize(axis=1).normalize(axis=0)

v_m_corpus = v_y_corpus.normalize(axis=1, keep_magnitude=True)

_ = v_ny_corpus.stats()


# %% [markdown]
# Compute (ordinary least square) linear regression on corpus nomalized by each years volume, but keeping the magnitude.callable
# The first year's values are kept, while rest of the years are scaled based on each year's token count compared to first years token count.

# %%

df_fits = gof.compute_goddness_of_fits_to_uniform(v_y_corpus)
df_fits.sort_values('l2_norm', ascending=False).head()


# %% [markdown]
# Two axes:
# https://stackoverflow.com/questions/25199665/one-chart-with-two-different-y-axis-ranges-in-bokeh
#

# %%


def plot_word_distribution(x_corpus, df):  # pylint: disable=too-many-locals

    tokens = list(df.token)

    colors = itertools.cycle(bokeh.palettes.Dark2[8])

    min_year = x_corpus.documents.year.min()
    max_year = x_corpus.documents.year.max()

    years = [str(y) for y in range(min_year, max_year + 1)]

    tokens_data = {token: x_corpus.get_word_vector(token) for token in tokens}

    tokens_data['year'] = years
    tokens_data['current'] = tokens_data[tokens[0]]

    source = bokeh.models.ColumnDataSource(tokens_data)

    # for token in tokens:
    #    color = next(colors)
    #    p.circle(x='year', y=token, size=5,source=source, color=color)
    #    p.line(x='year', y=token, source=source, color=color)

    color = next(colors)

    plot = figure(plot_width=800, plot_height=400, title="Word distribution", x_range=years, name="token_plot_01")

    plot.xgrid.grid_line_color = None
    plot.xaxis.axis_label = "Year"
    plot.xaxis.major_label_orientation = 1.2

    _ = plot.circle(x='year', y='current', size=5, source=source, color=color)
    _ = plot.line(x='year', y='current', source=source, color=color)

    # line_source = bokeh.models.ColumnDataSource({'x0': [], 'y0': [], 'x1': [], 'y1': []})
    # sr = p.segment(x0='x0', y0='y0', x1='x1', y1='y1', color='olive', alpha=0.6, line_width=3, source=line_source, )

    slider = bokeh.models.Slider(start=0, end=len(tokens) - 1, value=0, step=1, title="Word")
    next_button = bokeh.models.Button(label=">>")

    token_vectors = [tokens_data[w] for w in tokens]

    update_plot = bokeh.models.CustomJS(
        args=dict(source=source, tokens=tokens, token_vectors=token_vectors, title=plot.title),
        code="""
        try {
            // var plot = Bokeh.documents[0].get_model_by_name('token_plot_01');
            const data = source.data;
            const token_index = cb_obj.value;
            const y = data['current'];
            for (let i = 0; i < y.length; i++)
                y[i] = token_vectors[token_index][i];
            div.text = tokens[token_index];
            title.text = tokens[token_index].toUpperCase();
            //div.text = plot.title;
            source.change.emit();
        } catch (ex) {
            div.text = ex.toString();
        }
    """,
    )

    next_plot = bokeh.models.CustomJS(
        args=dict(slider=slider),
        code="""
        try {
            slider.value = slider.value + 1;
            slider.trigger('change');
        } catch (ex) {
            div.text = ex.toString();
        }
    """,
    )
    # find_token = CustomJS(args=dict(title=plot.title, token2id), code="""
    #     token = cb_obj.value
    # """)
    # text = TextInput(title='Enter title', value='my sine wave', callback=update_title)

    div = bokeh.models.Div(text="", width=400, height=10)
    # update_plot.args['div'] = div
    # update_plot.args['plot'] = plot
    slider.js_on_change('value', update_plot)
    next_button.js_on_click(next_plot)

    w = bokeh.layouts.grid([div, next_button, plot, slider])  # , sizing_mode='stretch_both')

    return w


widget = plot_word_distribution(v_ny_corpus, df_fits.head(2000))
output_file('word_browse.html', title='change title')
show(widget)


# %%

chi2_stats, chi2_pvalues = list(
    zip(*[gof.gof_chisquare_to_uniform(v_m_corpus.data[:, i]) for i in range(0, v_m_corpus.data.shape[1])])
)
dx = pd.DataFrame({'chisquare': chi2_stats, 'pvalues': chi2_pvalues})
dx.chisquare.hist(bins=1000)



import glob
import os
import types

import ipywidgets as widgets
import penelope.corpus.vectorized_corpus as vectorized_corpus
from IPython.display import display
from penelope.utility import get_logger

logger = get_logger()


def get_corpus_tags(corpus_folder):

    filenames = [os.path.basename(x) for x in glob.glob(os.path.join(corpus_folder, "*_vectorizer_data.pickle"))]
    tags = [x[0 : len(x) - len("_vectorizer_data.pickle")] for x in filenames]
    return tags


def load_vectorized_corpus(corpus_folder, corpus_tag, n_count, n_top, normalize_axis=None, year_range=(1922, 1989)):

    try:
        year_filter = lambda x: year_range[0] <= x['year'] <= year_range[1]
        x_corpus = (
            vectorized_corpus.VectorizedCorpus.load(tag=corpus_tag, folder=corpus_folder)
            .filter(year_filter)
            .group_by_year()
            .slice_by_n_count(n_count)
            .slice_by_n_top(n_top)
        )

        for axis in normalize_axis or []:
            x_corpus = x_corpus.normalize(axis=axis, keep_magnitude=False)

        return x_corpus

    except Exception as ex:
        logger.exception(ex)
        return None


def display_gui(corpus_folder, container=None):

    corpus_tags = get_corpus_tags(corpus_folder)

    if len(corpus_tags) == 0:
        logger.info("Please install at least one vectorized corpus")
        return None

    year_range = [1922, 1989]
    normalize_options = {'None': [], 'Over year': [0], 'Over word': [1], 'Over year and word': [0, 1]}
    gui = types.SimpleNamespace(
        corpus_tag=widgets.Dropdown(
            description="Corpus",
            options=corpus_tags,
            value=corpus_tags[0],
            layout=widgets.Layout(width='420px', color='green'),
        ),
        n_min_count=widgets.IntSlider(
            description="Min count", min=100, max=10000, value=500, layout=widgets.Layout(width='400px', color='green')
        ),
        n_top_count=widgets.IntSlider(
            description="Top count",
            min=100,
            max=1000000,
            value=5000,
            layout=widgets.Layout(width='400px', color='green'),
        ),
        normalize=widgets.Dropdown(
            description="Normalize",
            options=normalize_options,
            value=[0],
            disabled=False,
            layout=widgets.Layout(width='420px', color='green'),
        ),
        load=widgets.Button(description="Load", disabled=False, layout=widgets.Layout(width='80px', color='green')),
        year_range=widgets.IntRangeSlider(
            value=year_range, min=year_range[0], max=year_range[1], step=1, description='Period:'
        ),
        output=widgets.Output(layout=widgets.Layout(width='500px')),
    )

    def load(*args):  # pylint: disable=unused-argument

        gui.output.clear_output()

        with gui.output:

            gui.load.disabled = True
            gui.load.description = 'Wait...'
            gui.corpus_tag.disabled = True

            container.corpus = load_vectorized_corpus(
                corpus_folder,
                gui.corpus_tag.value,
                n_count=gui.n_min_count.value,
                n_top=gui.n_top_count.value,
                normalize_axis=gui.normalize.value,
                year_range=gui.year_range.value,
            )

            gui.load.disabled = False
            gui.corpus_tag.disabled = False
            gui.load.description = 'Load'

        # gui.output.clear_output()

        # with gui.output:

        #    logger.exception("Corpus loaded.")

    gui.load.on_click(load)

    display(
        widgets.HBox(
            [
                widgets.VBox([gui.corpus_tag, gui.normalize, gui.n_min_count, gui.n_top_count, gui.year_range]),
                widgets.VBox([gui.load]),
                gui.output,
            ]
        )
    )

    return gui



import types
import warnings

import ipywidgets as widgets
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display

from notebooks.common import TopicModelContainer, filter_document_topic_weights

logger = utility.get_logger()
# from beakerx import *
# from beakerx.object import beakerx
# beakerx.pandas_display_table()

warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)


def display_documents(
    inferred_topics: topic_modelling.InferredTopicsData, filters, threshold=0.0, output_format='Table', n_top=500
):

    document_topic_weights = filter_document_topic_weights(
        inferred_topics.document_topic_weights, filters=filters, threshold=threshold
    )

    if len(document_topic_weights) == 0:
        print('No data to display for this topic and threshold')
        return

    if output_format == 'Table':
        document_topic_weights = (
            document_topic_weights.drop(['topic_id'], axis=1)
            .set_index('document_id')
            .sort_values('weight', ascending=False)
            .head(n_top)
        )
        document_topic_weights.index.name = 'id'
        display(document_topic_weights)


def display_gui(state: TopicModelContainer):

    text_id = 'topic_document_text'

    gui = types.SimpleNamespace(
        n_topics=state.num_topics,
        text_id=text_id,
        text=widgets_utils.text_widget(text_id),
        topic_id=widgets.IntSlider(
            description='Topic ID', min=0, max=state.num_topics - 1, step=1, value=0, continuous_update=False
        ),
        n_top=widgets.IntSlider(description='#Docs', min=5, max=500, step=1, value=75),
        threshold=widgets.FloatSlider(
            description='Threshold', min=0.0, max=1.0, step=0.01, value=0.20, continues_update=False
        ),
        output_format=widgets.Dropdown(
            description='Format', options=['Table'], value='Table', layout=widgets.Layout(width="200px")
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0),
        output=widgets.Output(),
        prev_topic_id=None,
        next_topic_id=None,
    )

    gui.prev_topic_id = widgets_utils.button_with_previous_callback(gui, 'topic_id', state.num_topics)
    gui.next_topic_id = widgets_utils.button_with_next_callback(gui, 'topic_id', state.num_topics)

    def on_topic_change_update_gui(topic_id: int):

        topic_token_weights = state.inferred_topics.topic_token_weights
        if gui.n_topics != state.num_topics:
            gui.n_topics = state.num_topics
            gui.topic_id.value = 0
            gui.topic_id.max = state.num_topics - 1

        if len(topic_token_weights[topic_token_weights.topic_id == topic_id]) == 0:
            tokens = ["Topics has no significant presence in any documents in the entire corpus"]
        else:
            tokens = topic_modelling.get_topic_title(topic_token_weights, topic_id, n_tokens=200)

        gui.text.value = 'ID {}: {}'.format(topic_id, tokens)

    def update_handler(*_):

        gui.output.clear_output()

        with gui.output:

            on_topic_change_update_gui(gui.topic_id.value)

            display_documents(
                inferred_topics=state.inferred_topics,
                filters=dict(topic_id=gui.topic_id.value),
                threshold=gui.threshold.value,
                n_top=gui.n_top.value,
                output_format=gui.output_format.value,
            )

    gui.topic_id.observe(update_handler, names='value')
    gui.threshold.observe(update_handler, names='value')
    gui.n_top.observe(update_handler, names='value')
    gui.output_format.observe(update_handler, names='value')

    display(
        widgets.VBox(
            [
                widgets.HBox(
                    [
                        widgets.VBox(
                            [
                                widgets.HBox([gui.prev_topic_id, gui.next_topic_id]),
                                gui.progress,
                            ]
                        ),
                        widgets.VBox([gui.topic_id, gui.threshold, gui.n_top]),
                        widgets.VBox([gui.output_format]),
                    ]
                ),
                gui.text,
                gui.output,
            ]
        )
    )

    update_handler()
